#!/usr/bin/tclsh

#---------------#
# FUNCTIONS     #
#---------------#
proc ChangeToStartTerminal {} {
#######---ChangeToStartTerminal---######
# change to the console sax was 
# started
#
global var
catch { exec chvt $var(StartConsole) }
}

proc FbColor { } {
####----FbColor---######
# get framebuffer color depth
#
 global var
 set geometry "8";

 catch { set geometry  [ eval exec { $var(FbSet) | grep geometry } ] }
 foreach i $geometry {
  set geometry $i;
 }
 set geometry [ string trim $geometry ]
 return $geometry
}

proc InstallFvwmConfig {} {
###--InstallFvwmConfig---#####
# this function copies the fvwm2rc to $HOME
# for running a windowmanager
#
global var
if { [ file exists $var(FvwmConfig) ] } {
 catch { exec mv $var(FvwmConfig) $var(FvwmStoreConfig) }
}
if { [ file exists $var(Fvwm2Config) ] } {
 catch { exec mv $var(Fvwm2Config) $var(Fvwm2StoreConfig) }
}
if { [ file exists $var(Fvwm95Config) ] } {
 catch { exec mv $var(Fvwm95Config) $var(Fvwm95StoreConfig) }
}
catch { exec cp $var(FvwmSaXConfig) $var(FvwmConfig) }
catch { exec cp $var(FvwmSaXConfig) $var(Fvwm2Config) }
catch { exec cp $var(FvwmSaXConfig) $var(Fvwm95Config) }
catch { exec sync }
}

proc RestoreFvwmConfig {} {
####--RestoreFvwmConfig--###########
# this function resores an old fvwm2rc file
#
global var
catch { exec rm -f $var(FvwmConfig) }
catch { exec rm -f $var(Fvwm2Config) }
catch { exec rm -f $var(Fvwm95Config) }
if { [ file exists $var(FvwmStoreConfig) ] } {
 catch { exec mv $var(FvwmStoreConfig) $var(FvwmConfig) }
} 
if { [ file exists $var(Fvwm2StoreConfig) ] } {
 catch { exec mv $var(Fvwm2StoreConfig) $var(Fvwm2Config) }
}
if { [ file exists $var(Fvwm95StoreConfig) ] } {
 catch { exec mv $var(Fvwm95StoreConfig) $var(Fvwm95Config) }
}
}

proc GetStartTerminal {} {
####--GetStartTermninal--######
# try to find out which is the 
# start console
#
global var
set console 0
catch { set console [ exec tty ] }
set console [ string range $console end end ]

set var(StartConsole) $console
}

proc Usage {} {
#####--Usage---#########
# Explain the options
#
puts "usage: SaX \[ options \]"
puts "options:"
puts "   \[ -m | --memory < Memory, extension \[k,K|m,M|g,G\] > \]"
puts "   \[ -r | --ramdac < Ramdac Speed > ]"
puts "   \[ -s | --server < X Server > ]"
puts "   \[ -d | --device < Mouse Device > ]"
puts "     If Mouse is not avaliable use -d /dev/null"
puts "   \[ -p | --prot  < Mouse Protocol > \]"
puts "   \[ -h | --hsync < Hsync range > \]"
puts "   \[ -v | --vsync < Vsync range > \]"
puts "   \[ -l | --lang  < Language > \]"
puts "     possible languages:"
puts "      < --lang  de    -> German     >"
puts "      < --lang  en    -> English    >"
puts "      < --lang  es    -> Spanish    >"
puts "      < --lang  fr    -> French     >"
puts "      < --lang  el    -> Greek      >"
puts "      < --lang  id    -> Indonesia  >"
puts "      < --lang  it    -> Italian    >"
puts "      < --lang  hu    -> Hungarian  >"
puts "      < --lang  nl    -> Dutch      >"
puts "      < --lang  pl    -> Polish     >"
puts "      < --lang  pt    -> Portuguese >"
puts "      < --lang  pt_BR -> Brazilian  >"
puts "      < --lang  br    -> Breton     >"
puts "      < --lang  ro    -> Romanian   >"
puts "      < --lang  ru    -> Russian    >"
puts "      < --lang  sk    -> Slovak     >"          
puts "      < --lang  ca    -> Catalan    >"
puts "      < --lang  cs    -> Czech      >"
puts "      < --lang  ja    -> Japanese   >"
puts "      < --lang  ko    -> Korean     >"
puts "   \[ -w | --wm    < Set Windowmanager, none for switch off > \]"
puts "     Note: Usage only for fvwm based windowmanagers !"
puts "   \[ -a | --auto  < Resolution > \]"
puts "   \[ -o | --out   < off > \]"
puts "     Oppress any terminal output"
puts "   \[ -g | --gpm   < on | off > \]"
puts "     If you want to use gpm as repeater use -g on"
puts "     the default value is off"
puts "   \[ -C | --chipset < set the chipset > \]"
puts "   \[ -O | --options < set a comma separated option list > \]"
puts "   \[ -f | --force   < startup > \]"
puts "     this will force the execution of SaX on a still"
puts "     running X-Server. Note: If there is no Server up"
puts "     the SaX startup will fail\n"
puts "     Note: Auto configuration is still experimental"
puts "   \[ -M | --model   < XkbModel  > \]"
puts "   \[ -L | --layout  < XkbLayout > \]"
puts "   \[ -R | --rules   < XkbRules  > \]"
}

proc CleanAll {} {
####--CleanAll--#####
# this function cleans all tmp dirs
# after exit
#
#-------------------------------#
# remove sax temps              #
# and restore start config      #
#-------------------------------#
global var
catch { exec cp $var(ConfigOrig) $var(ConfigRun) }
catch { exec rm -rf $var(TmpSax) }
catch { exec rm -rf /tmp/.sax_dir }
catch { WriteToLog "SaX Complete" }
catch { WriteToLog "------------" }
catch { WriteToLog "   removed $var(TmpSax)" }
catch { WriteToLog "   removed /tmp/.sax_dir" }
catch { WriteToLog "   refreshed $var(ConfigRun) to $var(ConfigOrig)" }
}

proc EvaluateAutoConfig {} {
 global OptionList
 if { $OptionList(--auto) == "" } {
  return "-1"
 }
 #
 # set auto resolution
 # and the frequencies
 #
 set dummy ""
 set X     ""
 set Y     ""
 if { [ regexp -- {(.*)[xX](.*)} $OptionList(--auto) dummy X Y ] == 0 } {
  puts "SaX: wrong value $OptionList(--auto) for --auto"
  CleanAll 
  exit 0
 }
 set error [ IntegerTest $X ]
 if { $error == "-1" } {
  puts "SaX: wrong value $OptionList(--auto) for --auto"
  CleanAll
  exit 0
 } 
 set error [ IntegerTest $Y ]
 if { $error == "-1" } {
  puts "SaX: wrong value $OptionList(--auto) for --auto"
  CleanAll
  exit 0
 }
 set index 300
 catch { set index [ expr ( $X \* $Y ) / 1024 ] }
 if { $index <= 300 } {
  set OptionList(--hsync) "30-33"
  set OptionList(--vsync) "55-70"
 } elseif { $index <= 500 } {
  set OptionList(--hsync) "30-40"
  set OptionList(--vsync) "55-90"
 } else {
  set OptionList(--hsync) "30-60"
  set OptionList(--vsync) "55-100"
 }

 #
 # set ramdac speed
 #
 if { $OptionList(--ramdac) != "" } {
  set error [ IntegerTest $OptionList(--ramdac) ]
  if { $error == "-1" } {
   puts "SaX: wrong value $OptionList(--ramdac) for --ramdac"
   CleanAll
   exit 0
  }
  if { $OptionList(--ramdac) <= "0" } {
   puts "SaX: non possible ramdac speed $OptionList(--ramdac)"
   CleanAll
   exit 0
  }
 }

 #
 # set sync ranges 
 #
 if { $OptionList(--hsync) != "" } {
  set dummy ""
  set left  ""
  set right ""
  if { [regexp -- {(.*)\-(.*)} $OptionList(--hsync) dummy left right] == 0 } {
   puts "SaX: wrong range $OptionList(--hsync) for --hsync"
   CleanAll
   exit 0
  }
  set error [ IntegerTest $left ]
  if { $error == "-1" } {
   puts "SaX: wrong range $OptionList(--hsync) for --hsync"
   CleanAll
   exit 0
  }
  set error [ IntegerTest $right ]
  if { $error == "-1" } {
   puts "SaX: wrong range $OptionList(--hsync) for --hsync"
   CleanAll
   exit 0
  }
  if { $left > $right } {
   puts "SaX: wrong range $OptionList(--hsync) for --hsync"
   CleanAll
   exit 0
  }
 }
 if { $OptionList(--vsync) != "" } {
  set dummy ""
  set left  ""
  set right ""
  if { [regexp -- {(.*)\-(.*)} $OptionList(--vsync) dummy left right] == 0 } {
   puts "SaX: wrong range $OptionList(--vsync) for --vsync"
   CleanAll
   exit 0
  }
  set error [ IntegerTest $left ]
  if { $error == "-1" } {
   puts "SaX: wrong range $OptionList(--vsync) for --vsync"
   CleanAll
   exit 0
  }
  set error [ IntegerTest $right ]
  if { $error == "-1" } {
   puts "SaX: wrong range $OptionList(--vsync) for --vsync"
   CleanAll
   exit 0
  }
  if { $left > $right } {
   puts "SaX: wrong range $OptionList(--vsync) for --vsync"
   CleanAll
   exit 0
  }
 }
}

proc IntegerTest {item} {
set error "-1"
catch { set error [ expr $item \* 1 ] }
if { $error != -1 } {
 return "0"
} 
return "-1"
}

proc WriteToLog {line} {
global var
set handle [ open $var(StartLog) "a" ] 
puts $handle $line 
close $handle
}

proc StartServer {disp realname device protocol switch} {
#---------------#
# variables     #
#---------------#
global var
global OptionList
set filename $var(InfoPath)
set file     "Info_$realname"
set filename $filename$file
if { [ string tolower $realname ] == "xf86_fbdev" } {
 set depth [ FbColor ]
} else {
 set depth 8
}

catch { exec $var(RunServer) $realname $device $protocol $switch $depth }
#------------------#
# start wm if set  #
#------------------#
if { $OptionList(--wm) != "" } {
 if { $OptionList(--lang) == "" } {
  catch { set ipid [ exec $var(SaxIntro) -display $disp & ] }
  after 2000
 } else {
  set language $OptionList(--lang)
  catch { set ipid [ exec $var(SaxIntro) -l $language -display $disp & ] }
  after 2000
 }
 InstallFvwmConfig
 catch { exec $var(Fvwm) -display $disp & }
}

#--------------#
# run intro    #
#--------------#
if { $OptionList(--lang) == "" } {
  catch { set ipid [ exec $var(SaxIntro) -display $disp & ] }
  after 2000
} else {
  set language $OptionList(--lang)
  catch { set ipid [ exec $var(SaxIntro) -l $language -display $disp & ] }
  after 2000
}

#----------------#
# set background #
#----------------#
catch { exec $var(SetBackground) $realname }

return ipid
}


proc ProveOnRoot {} {
#--------------#
# variables    #
#--------------#
set ID  "nonroot"

catch { eval { exec whoami > /tmp/.sax_dir/user } }
set handle [ open "/tmp/.sax_dir/user" "r" ]
gets $handle ID
close $handle
catch { eval { exec rm -f /tmp/.sax_dir/user } }
if { $ID != "root" } {
 puts "SaX: only root can start SaX"
 puts -nonewline "SaX: allow root to access server..."
 flush stdout
 set error [ catch { eval { exec xhost + localhost >/dev/null 2>/dev/null } } ]
 if { $error == "0" } {
  puts "done"
 } else {
  puts "no_server"
 }
 puts "SaX: please switch to root and start again"
 CleanAll 
 exit "0"
}
}


proc GetOptions {iOptionList} {
#####--GetOptions--#######
# This function read all options
# possible are:
# --auto    X*Y
# --server  Xserver
# --hsync   hsyncfreq
# --vsync   vsyncfreq
# --memory  memory
# --ramdac  DAC-speed
# --lang    id
# --prot    mouseprotocol
# --device  mousedevice
# --out     on | off
# --wm      fvwm | fvwm95
# --gpm     on | off
# --chipset string
# --options string
# --vendor  Vendor        (hidden option)
# --name    name of card  (hidden option)   
# --force   identifier
# --rules   XkbRules
# --model   XkbModel
# --layout  XkbLayout
#

upvar $iOptionList option
global argv

set    argcount     "0"
set    arg          ""
set    option(--auto)     ""     
set    option(--server)   ""
set    option(--hsync)    "" 
set    option(--vsync)    ""
set    option(--memory)   ""
set    option(--ramdac)   ""
set    option(--lang)     "" 
set    option(--prot)     ""
set    option(--device)   ""
set    option(--out)      ""
set    option(--wm)       ""
set    option(--gpm)      ""
set    option(--chipset)  ""
set    option(--options)  ""
set    option(--name)     ""
set    option(--vendor)   ""
set    option(--force)    ""
set    option(--rules)    ""
set    option(--model)    ""
set    option(--layout)   ""
    
while { 1 } {
        
 set arg [ lindex $argv $argcount ]
 incr argcount
 switch -glob -- "$arg" {
            -R             -
            --rules        {
               
               set arg "--rules"
            }

            -M             -
            --model        {

               set arg "--model"
            } 

            -L             -
            --layout       {

               set arg "--layout"
            }

            -f             -
            --force        {

               set arg "--force"
            }

            -V             -
            --vendor       {

               set arg "--vendor"
            }

            -N             -
            --name         {

               set arg "--name"
            }

            -C             -
            --chipset      {

               set arg "--chipset"
            }

            -O             -
            --options      {

               set arg "--options"
            }

            -g             -
            --gpm          {

               set arg "--gpm"
            } 
       
            -w             -
            --wm           {
   
               set arg "--wm"
            } 
            
            -o             -
            --out          {

                set arg "--out"
            }  

            -s             -
            --server       {
                
                set arg "--server"
            }
            
            -p             -
            --prot         {

                set arg "--prot"
            }

            -d             -
            --device       {

                set arg "--device"
            }

            -m             -
            --memory       {
                
                set arg "--memory"
            }
            
            -a             -
            --auto         {
                
                set arg "--auto"
            }
            
            -l             -
            --lang         { 
                
                set arg "--lang"
            }
            
            -r             -
            --ramdac       {
                
                set arg "--ramdac"
            } 
 
            -h             -
            --hsync        {

                set arg "--hsync"
            }
 
            -v             -
            --vsync        {

                set arg "--vsync"
            }
            
            -*           {
                
                puts "SaX: $arg is not a valid option."
                Usage
                CleanAll 
                exit "0"
            }
            
            default { 

                break

            }  
 }
 set option($arg) [ lindex $argv $argcount ]
 incr argcount
        
 if { $option($arg) == "" } {
            
  puts "SaX: missing parameter to option $arg"
  Usage
  CleanAll
  exit "0"
 }
}
}

proc GetServerFile {alias} {
#------------#
# variables  #
#------------#
global var
set RealServer ""

cd $var(Xbin)
set Xserver [ string toupper $alias ]
foreach i [lsort [glob X*]] {
 if { [file isfile $i] } {
  set UpName [ string toupper $i ]
  if { [ regexp -- "\(.*\)$Xserver$" $UpName ] == 1 } {
   set  RealServer $i
  break
  }
 }
}
if { $RealServer == "" } {
 return "-1"
} else {
 return $RealServer
}

}

proc XtoGpm {protocol} {
###--XtoGpm---####
# this function returns the gpm
# protocol adapted to the given X protocol
#
set gtype()     ""
set gtype(mouseman)     mman
set gtype(microsoft)    ms
set gtype(mousesystems) msc
set gtype(logitech)     logi
set gtype(busmouse)     bm
set gtype(ps/2)         ps2
set gtype(intellimouse) ms3
set gtype(imps/2)       imps2

global gpmprotocol
set prot [ string tolower $protocol ]
foreach i [ array names gtype ] {
 if { $prot == $i } {
  set gpmprotocol $gtype($i)
  break
 }
}
}

proc GetGpmData {idev iprot ispeach} {
#-----------#
# Variables #
#-----------#
upvar $idev    device
upvar $iprot   protocol
upvar $ispeach speach
set gpmdata  ""
set mouse    ""

global gpmprotocol
set gpmprotocol "ms"

#---------------------------#
# Get the information from  #
# /etc/rc.config            #
#---------------------------#
if { [ file exists "/etc/rc.config" ] == 0 } {
  #
  # rc.config does not exist -> we set 
  # device on COM 1 and protocol to Microsoft
  #
  set device "/dev/ttyS0"
  set protocol "Microsoft"
  return "-1"
} 
catch { set gpmdata [ eval exec { cat /etc/rc.config | grep ^GPM_PARAM= } ] }
catch { set mouse   [ eval exec { cat /etc/rc.config | grep ^MOUSE= } ] }
catch { set speach  [ eval exec { cat /etc/rc.config | grep ^LANGUAGE= } ] }

#-------------------------------#
# set the language translation  #
#-------------------------------#
set speachtype(german)        de
set speachtype(english)       en
set speachtype(french)        fr
set speachtype(italian)       it
set speachtype(spanish)       es
set speachtype(brazilian)     br
set speachtype(breton)        bz
set speachtype(dutch)         nl
set speachtype(hungarian)     hu
set speachtype(polish)        pl
set speachtype(portuguese)    pt
set speachtype(slovak)        sk
set speachtype(romanian)      ro
set speachtype(greek)         el
set speachtype(indonesian)    id
set speachtype(russian)       ru
set speachtype(turkish)       tr
set speachtype(swedish)       sv
set speachtype(czech)         cs
set speachtype(japanese)      ja
set speachtype(korean)        ko

#-------------#
# set speach  # 
#-------------#
set speach [ string trim $speach ]
set dummy ""
set left  ""
set right ""
regexp -- {(.*)=(.*)} $speach dummy left right
set right [ string trim $right {"} ]

set found "no"
foreach i [ array names speachtype ] {
 if { $i == $right } {
   set found "yes"
   break
 }
}
if { $found == "yes" } {
 set speach $speachtype($right)
}

#------------------------------#
# set the available mousetypes #
#------------------------------#
global gpmtype
set gpmtype()      ""
set gpmtype(mman)  Mouseman
set gpmtype(ms)    Microsoft
set gpmtype(bare)  Microsoft
set gpmtype(msc)   MouseSystems
set gpmtype(sun)   MouseSystems
set gpmtype(logi)  Logitech
set gpmtype(bm)    BusMouse
set gpmtype(ps2)   PS/2
set gpmtype(ps/2)  PS/2
set gpmtype(ms3)   IntelliMouse
set gpmtype(imps2) IMPS/2

#--------------------------------#
# get the informations from      #
# gpmdata  and mouse             #
#--------------------------------#
set device ""
set protocol ""
#puts $gpmdata

set z "0"
set dummy ""
set left  "" 
regexp -- {(.*)GPM_PARAM=(.*)} $gpmdata dummy left gpmdata
set gpmdata [ string trim $gpmdata "\""]
foreach i $gpmdata {
 if { $i == "-t" } {
  set index [ expr $z + 1 ]
  set protocol [ lindex $gpmdata $index ]
  set gpmprotocol [ string trim $protocol ]
 }
 if { $i == "-m" } {
  set index [ expr $z +1 ]
  set device [ lindex $gpmdata $index ]
 }
 incr z
}

set dummy ""
set left  ""
set right ""
regexp -- {(.*)\=(.*)} $mouse dummy left right
if { $right == "" } {
 set device "/dev/ttyS0"
} else {
 set device [ string trim $right ]
 set device [ string trim $device {"} ] 
}


set protocol [ string trim $protocol ]
set protocol [ string trim $protocol {"} ]
set protocol [ string tolower $protocol ]
set found "no"
foreach i [ array names gpmtype ] {
 if { $i == $protocol } {
  set found "yes"
  break
 }
}
if { $found == "yes" } {
 set protocol $gpmtype($protocol)
} else {
 set protocol $gpmtype(ms)
}
if { $protocol == "" } {
 set protocol "Microsoft"
}

if { $protocol == "" } {
 set protocol "Microsoft"
}
if { $device == "" } {
 set device "/dev/ttyS0"
}

#puts $protocol
#puts $device
#puts $speach
}

proc ScanKeyboard { } {
 global var

 set kprobe ""
 catch { set kprobe [ exec $var(Kprobe) ] }

 set dummy  ""
 set model  ""
 set rules  ""
 set layout ""
 regexp -- {(.*):(.*):(.*)} $kprobe dummy rules model layout

 if {($rules == "(null)") || ($model == "(null)") || ($layout == "(null)")} {
  return "null"
 }

 set result [ list $rules $model $layout ]
 return $result
}


proc ScanMouse { } {
 global var

 set mprobe ""
 catch { set mprobe [ exec $var(Mprobe) ] }

 set dummy  ""
 set prot  ""
 set dev  ""
 regexp -- {(.*):(.*)} $mprobe dummy prot dev

 if {($prot == "") || ($dev == "")} {
  return "null"
 }

 set result [ list $prot $dev ]
 return $result
}

#-----------------------------#
#  MAIN                       #
#-----------------------------#

#----------------#
# prove on root  #
#----------------#
#
# Security
#
catch { exec rm -rf /tmp/.sax_dir }
set make_dir [ catch {eval {exec mkdir /tmp/.sax_dir} } ]
if { $make_dir != 0 } {
 puts "Error: temporary directory still exists! Aborting for security \
              reasons."
 CleanAll
 exit -1
}
ProveOnRoot

#---------------#
# set pathes    #
#---------------#
global var
set var(FbSet)              "/var/X11R6/sax/bin/fbset" 
set var(SetBackground)      "/var/X11R6/sax/bin/SetBackground"
set var(BackDark)           "/var/X11R6/sax/pixmaps/dark.xpm"
set var(BackNormal)         "/var/X11R6/sax/pixmaps/setup.xpm"
set var(Fvwm)               "/usr/X11R6/bin/fvwm2" 
set var(FvwmConfig)         "$env(HOME)/.fvwmrc"
set var(Fvwm2Config)        "$env(HOME)/.fvwm2rc"
set var(Fvwm95Config)       "$env(HOME)/.fvwm95rc"
set var(FvwmStoreConfig)    "$env(HOME)/.fvwmrc.saxsave"
set var(Fvwm2StoreConfig)   "$env(HOME)/.fvwm2rc.saxsave"
set var(Fvwm95StoreConfig)  "$env(HOME)/.fvwm95rc.saxsave"
set var(FvwmSaXConfig)      "/var/X11R6/sax/config/.fvwm2rc"
set var(InfoPath)           "/var/X11R6/sax/tmp/"
set var(RunServer)          "/var/X11R6/sax/bin/StartNewX"
set var(PrepareConfig)      "/var/X11R6/sax/bin/PrepConfig"
set var(PrepareFileSection) "/var/X11R6/sax/bin/PrepFileSection"
set var(TmpSax)             "/tmp/sax/"
set var(ConfigRun)          "/var/X11R6/sax/config/StartConfig"
set var(ConfigOrig)         "/var/X11R6/sax/config/StartConfig.orig"
set var(Xbin)               "/usr/X11R6/bin/"
set var(ScanPci)            "/var/X11R6/sax/bin/ScanPci"
set var(RunMainSax)         "/var/X11R6/sax/bin/RunMainSax"
set var(StartLog)           "/root/StartLog"
set var(ServerLog)          "/root/ServerLog"
set var(SaxIntro)           "/var/X11R6/sax/bin/intro.tcl"
set var(ScanIt)             "YES"
set var(StartConsole)       "0"
set var(SaxRc)              "/var/X11R6/sax/config/saxrc"
set var(Kprobe)             "/var/X11R6/sax/bin/kprobe"
set var(Mprobe)             "/var/X11R6/sax/bin/mprobe"

#----------------#
# Remove old log #
#----------------#
#if { [ file exists $var(StartLog) ] } {
# catch { exec rm -f $var(StartLog) }
#}

#-------------------#
# get start console #
#-------------------#
GetStartTerminal
WriteToLog "SaX Start Terminal"
WriteToLog "------------------"
WriteToLog "\tTerminal: $var(StartConsole)"
WriteToLog "\n"

#--------------#
# Get Options  #
#--------------#
global OptionList
GetOptions OptionList
WriteToLog "Options"
WriteToLog "-------"
foreach i [ array names OptionList ] {
 WriteToLog "\tOption: $i => $OptionList($i)"
}
WriteToLog "\n"

#--------------#
# set keyboard #
#--------------#
set kprobe [ ScanKeyboard ]
if { $OptionList(--rules) == "" } {
 if { $kprobe != "null" } {
  set OptionList(--rules) [ lindex $kprobe 0 ]
 }
}
if { $OptionList(--layout) == "" } {
 if { $kprobe != "null" } {
  set OptionList(--layout) [ lindex $kprobe 2 ]
 }
}
if { $OptionList(--model) == "" } {
 if { $kprobe != "null" } {
  set OptionList(--model) [ lindex $kprobe 1 ]
 }
}

#----------------#
# set language   #
#----------------#
set device   ""
set protocol ""
set speach   ""
GetGpmData device protocol speach
WriteToLog "Startup Language"
WriteToLog "----------------"

if { $OptionList(--lang) == "" } {
 if { [ file exists $var(SaxRc) ] } {
  set handle [ open $var(SaxRc) "r" ] 
  set Par_lang    [ gets $handle ]
  close $handle
  set dummy "" 
  set left  ""
  set right "" 
  regexp -- {(.*):(.*)} $Par_lang dummy left right
  set Par_lang $right
  set Par_lang [ string trim $Par_lang ]
  WriteToLog "\tLanguage set to $Par_lang ( saxrc )"
 }
 if { $speach != "" } {
  set Par_lang $speach
  set OptionList(--lang) $speach
  WriteToLog "\tLanguage set to $Par_lang ( rc.config )"
 }
} else {
 set Par_lang $OptionList(--lang)
 WriteToLog "\tLanguage set to $Par_lang ( option )"
}

#-------------------#
# set windowmanager #
#-------------------#
global var
if { $OptionList(--wm) != "" } {
 WriteToLog "Windowmanager"
 WriteToLog "-------------"
 WriteToLog "\t Set wm to: $OptionList(--wm)"
 set var(Fvwm) $OptionList(--wm)
}

#---------------------#
# trunc server name   #
#---------------------#
set dummy ""
set left  ""
set right ""
if { $OptionList(--server) != "" } {
 if {[ regexp -- {(.*)\_(.*)} $OptionList(--server) dummy left right ] != 0} {
  set OptionList(--server) $right
 }
}

#--------------------------------#
# test parameters of auto config #
#--------------------------------#
EvaluateAutoConfig

#-------------------------------------------#
# look for server if given by the option -s #
# or scan the server                        #
#-------------------------------------------#
if { $OptionList(--server) != "" } {

 set Xserver     [ string toupper $OptionList(--server) ]
 set RealServer  ""
 set var(ScanIt) "NO" 
 set RealServer [ GetServerFile $Xserver ]
 if { $RealServer == "-1" } {

  if { $OptionList(--auto) != ""} {
   puts "SaX: $Xserver server not installed"
   puts "SaX: for auto configuration please install the server first"
   CleanAll
   exit "0"
  } else {
   puts "SaX: server $Xserver is not installed"
   CleanAll
   exit "0"
  }

 }
 if { $OptionList(--auto) != ""} {
  puts "SaX: start auto configuration for $RealServer"
  set alias $Xserver
 } else {
  if { $OptionList(--out) != "off" } {
   puts "SaX: using $RealServer for configuration"
  }
  set alias $Xserver
 }

} else {

 set var(ScanIt) "YES"
 global env
 if { $OptionList(--out) != "off" } {
  puts -nonewline "SaX: searching card..."
  flush stdout
 }
 set alias ""
 catch { set alias [ eval exec { $var(ScanPci) -d sax } ] }
 if { $OptionList(--out) != "off" } {
  puts "done"
 }
 if { ($alias == "Failed") || ($alias == "") } {

  if { $OptionList(--auto) != "" } {
   puts "SaX: scanning failed"
   puts "SaX: please set the server manually using the -s option"
   puts "SaX: for example \[   sax -s svga   \]"
   CleanAll
   exit "0"
  } else {
   if { $OptionList(--out) != "off" } {
    set arch "i686"
    catch { set arch [ exec uname -m ] }
    if { $arch == "alpha" } {
     puts "SaX: scanning failed"
     puts "SaX: please set the server manually using the -s option" 
     puts "SaX: for example \[   sax -s svga   \]"
     CleanAll
     exit "0"
    } else {
     puts "SaX: scanning failed switching to fallback server"
    }
   }

   # We could not detect a server for this card
   # first try framebuffer than vga
   # --------------------------------
   set slist [ list FBDEV XSUN XSUN24 XSUNMONO ]
   foreach n $slist {
    set alias $n 
    set RealServer [ GetServerFile $alias ]
    if { $RealServer == "-1" } {
     #puts "SaX: trying $n not installed... next" 
    } else {
     break
    }
   }
   if { $RealServer == "-1" } {
    #puts "SaX: no framebuffer server installed... trying vga" 
    set alias VGA16
    set RealServer [ GetServerFile $alias ]
    if { $RealServer == "-1" } {
     #puts "SaX: vga server not installed... abort"
     CleanAll
     exit "0"
    }
   }

   if { $OptionList(--out) != "off" } {
    puts "SaX: using $RealServer for configuration" 
   }
  }

 } else {
  set RealServer [ GetServerFile $alias ]
  if { $RealServer == "-1" } {

   if { $OptionList(--auto) != "" } {
    puts "SaX: $alias server not installed"
    puts "SaX: for auto configuration please install the server first"
    CleanAll
    exit "0"
   } else {
    if { $OptionList(--out) != "off" } {
     puts "SaX: $alias server not installed"
     puts "SaX: switching to VGA16 server"
    }
    set alias VGA16
    set RealServer [ GetServerFile $alias ]
    if { $RealServer == "-1" } {
     puts "SaX: VGA16 server not installed abort"
     CleanAll
     exit "0"
    }
    if { $OptionList(--out) != "off" } {
     puts "SaX: using $RealServer for configuration" 
    }
   }

  } else {

   if { $OptionList(--auto) != "" } {

    puts "SaX: start auto configuration for $RealServer"

   } else {

    if { $OptionList(--out) != "off" } {
     puts "SaX: using $RealServer for configuration"
    }

   } 
   if { $OptionList(--out) != "off" } { 
    puts "SaX: if not correct set server manually using the -s option"
    puts "SaX: go up again within 8 seconds \[ Ctrl-c to abort \]"
    set count 0
    puts -nonewline "sax: "
    flush stdout
    while { 1 } {
     after 1000
     puts -nonewline .
     flush stdout 
     if { $count == "8" } { 
      break
     }
     incr count
    }
    puts "\n"
   } 

  }
 }
 set OptionList(--server) $alias
}

# if we are using the sun servers...
# -----------------------------------
# get fbdev resolution...
# ------------------------
set name [ string tolower $RealServer ]
if { [ regexp -- {.*sun.*} $name ] == 1 } {
 set res ""
 catch { set res [ eval exec { /var/X11R6/sax/bin/fbset | grep ^mode } ] }
 foreach i $res {
  set res $i;
 }
 set dummy ""
 set left  "640"
 set right "480"
 regexp -- {(.*)x(.*)-(.*)} $res dummy left right
 set res "[set left]x[set right]"
 global OptionList

 set OptionList(--auto)   $res
 set OptionList(--server) $alias

 if { $OptionList(--out) == "" } {
  puts ""
  puts "  creating a sun configuration automatically"
  puts "  After the SaX configuration just type in:"
  puts ""
  puts "  startx"
  puts ""
 }
}

WriteToLog "Server"
WriteToLog "------"
WriteToLog "\tAlias name: $alias"
WriteToLog "\tReal file name: $RealServer"
WriteToLog "\n" 

#----------------------#
# read rc.config stuff #
#----------------------#
set device   ""
set protocol ""
set speach   ""
if { ($OptionList(--prot) == "") && ($OptionList(--device) == "") } {
 GetGpmData device protocol speach
 set OptionList(--device) $device
 set OptionList(--prot)   $protocol
 WriteToLog "rc.config stuff"
 WriteToLog "---------------"
 WriteToLog "\tMouse Device: $device"
 WriteToLog "\tMouse Protocol: $protocol"
 WriteToLog "\n"
} else {
 GetGpmData device protocol speach
 if { ($OptionList(--prot) != "") && ($OptionList(--device) != "") } {
  set device   $OptionList(--device)
  set protocol $OptionList(--prot)
  XtoGpm $protocol
 } else {
  if { $OptionList(--device) != "" } {
   set device $OptionList(--device)
   set OptionList(--prot) $protocol
  }
  if { $OptionList(--prot) != "" } {
   set protocol $OptionList(--prot)
   set OptionList(--device) $device
   XtoGpm $protocol
  }
 }

 WriteToLog "mouse settings given through command line option"
 WriteToLog "------------------------------------------------"
 WriteToLog "\tMouse Device: $device"
 WriteToLog "\tMouse Protocol: $protocol"
}

#--------------------------------#
# look up mouse using libhd      #
# -------------------------------#
set mprobe [ ScanMouse ]
if { $mprobe != "null" } {
 set protocol [ lindex $mprobe 0 ]
 set device   [ lindex $mprobe 1 ]
}

WriteToLog "mouse settings via libhd"
WriteToLog "------------------------"
WriteToLog "\tMouse Device: $device"
WriteToLog "\tMouse Protocol: $protocol"

#---------------------------#
# create temporar SaX dir   #
# and set up the mouse link #
#---------------------------#
catch { exec rm -rf $var(TmpSax) }
set make_dir [ catch {eval {exec mkdir $var(TmpSax)} } ]
if { $make_dir != 0 } {
 puts "Error: temporary directory still exists! Aborting for security \
               reasons."
 CleanAll
 exit -1
}

WriteToLog "Temp directory"
WriteToLog "--------------"
WriteToLog "\tmake tmp dir $var(TmpSax)"

# security
# ---------
set chmod_dir [ catch {eval {exec chmod 755 $var(TmpSax)} } ]
if { $chmod_dir != 0 } {
 puts "Error: I'm not owning $var(TmpSax)! Aborting for security reasons."
 CleanAll
 exit -1
}
cd $var(TmpSax)
exec rm -f mouse 
exec ln -s $device mouse
WriteToLog "Mouse link"
WriteToLog "----------"
WriteToLog "\tmake mouse link \"ln -s $device mouse\""
WriteToLog "\n"

#-------------------------------#
# prepare start config for sax  #
#-------------------------------#
set alias [ string tolower $alias ]
if { $alias != "vga16" } {
 catch { exec $var(PrepareConfig) $protocol chipdisable $Par_lang $OptionList(--chipset) $OptionList(--options)}
 catch { exec $var(PrepareFileSection) -l $Par_lang }
} else {
 catch { exec $var(PrepareConfig) $protocol chipgeneric $Par_lang $OptionList(--chipset) $OptionList(--options)}
 catch { exec $var(PrepareFileSection) -l $Par_lang } 
}

#--------------#
# get display  #
#--------------#
global env
if { [ info exists env(DISPLAY) ] == 0 } {
 set env(DISPLAY) ":0"
}  
if { $env(DISPLAY) == "" } {
 set env(DISPLAY) ":0"
}
set disp $env(DISPLAY)
set dummy ""
set left  ""
set right ""
regexp -- {(.*):(.*)} $disp dummy left right
set disp [ string range $right 0 0 ]
set disp [ expr $disp + 1 ]
set disp ":$disp"
WriteToLog "Display"
WriteToLog "-------"
WriteToLog "\tDisplay is: $disp"
WriteToLog "\n"

#----------------------------------#
# test if another Server is active #
#----------------------------------#
set pid ""
catch { set pid [ exec pidof $var(Xbin)$RealServer ] }

# fast hack to force startup with a new server...
# ------------------------------------------------
set pid ""

if { ($pid == "") && ($OptionList(--force) != "startup") } {
 
 if { $OptionList(--out) != "off" } {
  puts "SaX: start new $RealServer server"
 }
 WriteToLog "Server Setting"
 WriteToLog "--------------"
 WriteToLog "\tstart new $RealServer server"
 WriteToLog "\n"
 #
 # start new server
 #
 global OptionList
 global gpmprotocol
 if { $OptionList(--gpm) == "on" } {
  set devicelink $device
  set device                 "/dev/gpmdata"
  set OptionList(--device)   "/dev/gpmdata"
  set OptionList(--prot)     "MouseSystems"
  set ipid [ StartServer $disp $RealServer $devicelink $gpmprotocol on ]
 } else {
  set ipid [ StartServer $disp $RealServer $device $protocol off ]
 }
 #
 # run sax
 #
 set options ""
 foreach i [ array names OptionList ] {
  if { $OptionList($i) != "" } {
   set options [ linsert $options end $i ]
   set options [ linsert $options end $OptionList($i) ]
  }
 }
 #puts $options
 exec $var(RunMainSax) $options -display $disp

 #
 # kill intro and server
 #
 set handle [ open $var(TmpSax)pids "r" ] 
 set spid [ gets $handle ]
 close $handle
 catch { exec kill $ipid }
 catch { exec kill -15 $spid }
 ChangeToStartTerminal
 WriteToLog "Refresh Terminal"
 WriteToLog "----------------"
 WriteToLog "\tgo back to: $var(StartConsole)"
 WriteToLog "\n"
} else {

 if { $OptionList(--out) != "off" } { 
  if {  $OptionList(--force) == "startup" } {
   puts "SaX: option -f is set... force startup"
  } else {
   puts "SaX: run SaX on actual server"
  }
 }
 WriteToLog "Server Setting"
 WriteToLog "--------------"
 WriteToLog "\trun SaX on actual server"
 WriteToLog "\n"
 set options ""
 foreach i [ array names OptionList ] {
  if { $OptionList($i) != "" } {
   set options [ linsert $options end $i ]
   set options [ linsert $options end $OptionList($i) ] 
  }
 }
 #puts $options
 exec $var(RunMainSax) $options

}

#-------------------------------#
# remove sax temps              #
# and restore start config      #
#-------------------------------#
catch { exec cp $var(ConfigOrig) $var(ConfigRun) }
catch { exec rm -rf $var(TmpSax) }
catch { exec rm -rf /tmp/.sax_dir }
if { $OptionList(--wm) != "" } {
 RestoreFvwmConfig
}
WriteToLog "Ending"
WriteToLog "------"
WriteToLog "\tremoved $var(TmpSax)"
WriteToLog "\tremoved /tmp/.sax_dir"
WriteToLog "\trefreshed $var(ConfigRun) to $var(ConfigOrig)"
